package com.biganto.visual.roompark.presentation.screen.article

import android.text.Html
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.graphics.drawable.toDrawable
import androidx.core.os.bundleOf
import androidx.core.text.HtmlCompat
import butterknife.BindView
import butterknife.OnClick
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.google.android.material.floatingactionbutton.FloatingActionButton
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.BlurTransformation
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val ARTICLE_ID = "DIRECT_ARTICLE_ID_KEY"

class ArticleScreenController  :
    BigantoBaseController<ArticleScreenViewState
        , ArticleScreen
        , ArticleScreenPresenter>
    , ArticleScreen {


    constructor()

    constructor(id:Int):super(bundleOf(ARTICLE_ID to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: ArticleScreenPresenter

    @BindView(R.id.articleTitle)
    lateinit var title:MaterialTextView

    @BindView(R.id.articleHeaderBlock)
    lateinit var headerBlock: ViewGroup

    @BindView(R.id.articleContent)
    lateinit var contentView: MaterialTextView

    @BindView(R.id.articleCloseButton)
    lateinit var closeArticle: FloatingActionButton


    val blurPreview:ImageView by lazy {
       headerBlock.findViewById<ImageView>(R.id.articlePreviewBlurred)
    }

    val articlePreview:ImageView by lazy {
        headerBlock.findViewById<ImageView>(R.id.articlePreview)
    }

    val articleDate:MaterialTextView by lazy {
        headerBlock.findViewById<MaterialTextView>(R.id.articleDate)
    }

    val closeButton:FloatingActionButton by lazy {
        headerBlock.findViewById<FloatingActionButton>(R.id.articleCloseButton)
    }

    @OnClick(R.id.articleCloseButton)
    fun onCloseArticle(){
        handleBack()
    }



    private fun setToolbar(){
        toolBar.hideAll()
        toolBar.appBar.setExpanded(false,false)
//        toolBar.collapsingToolbarLayout.title = "ИЗБРАННОЕ"
//        toolBar.appBar.liftOnScrollTargetViewId = R.id.Article_cards_recycler_view
        toolBar.appBar.setLiftable(false)
        toolBar.appBarScrollable(false)
//        ArticleRecyclerView.isNestedScrollingEnabled = false
    }

    private fun bindRecycler() {

    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: ArticleScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticleScreenViewState.Idle -> render(viewState)
            is ArticleScreenViewState.ArticleLoaded -> render(viewState)
        }
    }

    private fun render(viewState: ArticleScreenViewState.Idle){

    }


    private fun render(viewState: ArticleScreenViewState.ArticleLoaded) {

        title.text = viewState.item.title
        contentView.text = Html.fromHtml(viewState.item.htmlBody,
            HtmlCompat.FROM_HTML_MODE_LEGACY, Html.ImageGetter { url ->
                Picasso.get()
                    .load(url)
                    .get()
                    .toDrawable(resources!!)
            },HtmlTa)
        articleDate.text = viewState.item.published.toString()

        Picasso.get()
            .load(viewState.item.previewUrl)
            .transform(BlurTransformation(activity,40,4))
            .into(blurPreview)

        Picasso.get()
            .load(viewState.item.previewUrl)
            .into(articlePreview)
    }

    private fun getComponent() = DaggerArticleScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(ARTICLE_ID))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_read_screen


}


public class HtmlTagHandler implements Html.TagHandler {
    @Override
    public void handleTag(boolean opening, String tag, Editable output, XMLReader xmlReader) {
        Log.e("Tag", tag);
        if (tag.startsWith("b") ) {
            Log.e("Tage", "Code tag encountered1");
            if (opening) {
                Log.e("Tage", "Code tag encountered1");
                output.setSpan(new TypefaceSpan("monospace"), output.length(), output.length(), Spannable.SPAN_MARK_MARK);
            } else {
                Log.e("Tage", "Code tag encountered2");
                Object obj = getLast(output, TypefaceSpan.class);
                int where = output.getSpanStart(obj);
                output.removeSpan(obj);
                output.setSpan(new TypefaceSpan("monospace"), where, output.length(), Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            }
        }
        else {
            Log.e("Tage", "Code Exited");
        }

    }

    private Object getLast(Editable text, Class kind) {

        Object[] objs = text.getSpans(0, text.length(), kind);
        if (objs.length == 0) {
            return null;
        } else {
            for (int i = objs.length; i > 0; i--) {
                if (text.getSpanFlags(objs[i - 1]) == Spannable.SPAN_MARK_MARK) {
                    return objs[i - 1];
                }
            }
            return null;
        }
    }